﻿#include "Win32Application.h"

#if _WIN32 || _WIN64

int FWin32Application::Run(HINSTANCE hInstance, int nCmdShow)
{
    if(FWin32Application* App = new FWin32Application())
    {
        App->SetAppInstance(hInstance);
        App->SetAppCmdShow(nCmdShow);
        App->SetAppTitle(L"Win32Application");
        
        if(App->Init())
        {
            App->Update();
            App->Destroy();
        }
    }
    
    return 0;
}

bool FWin32Application::Init()
{
   if(FApplication::Init())
   {
       //解析命令行
       ParseCommandLine();

       //初始化窗口
       InitializeWindow();

       //初始化渲染管线
       InitializePipeline();
       
       return true;
   }

    return false;
}

void FWin32Application::Update()
{
   
}

void FWin32Application::Destroy()
{
    FApplication::Destroy();
}

void FWin32Application::SetWindowSize(UINT Width, UINT Height)
{
    AppWidth = Width;
    AppHeight= Height;

    RECT windowRect = { 0, 0, static_cast<LONG>(GetWidth()), static_cast<LONG>(GetHeight()) };
    AdjustWindowRect(&windowRect, WS_OVERLAPPEDWINDOW, FALSE);
}

void FWin32Application::ParseCommandLine()
{
    int argc;
    LPWSTR* argv = CommandLineToArgvW(GetCommandLineW(), &argc);
    LocalFree(argv);
}

void FWin32Application::InitializeWindow()
{
    // Initialize the window class.
    WNDCLASSEX windowClass = { 0 };
    windowClass.cbSize = sizeof(WNDCLASSEX);
    windowClass.style = CS_HREDRAW | CS_VREDRAW;
    windowClass.lpfnWndProc = WindowProc;
    windowClass.hInstance = GetAppInstance();
    windowClass.hCursor = LoadCursor(NULL, IDC_ARROW);
    windowClass.lpszClassName = L"xyzEngine";
    RegisterClassEx(&windowClass);

    SetWindowSize(GetWidth(), GetHeight());

    Hwnd = CreateWindow(
        windowClass.lpszClassName,
        GetAppTitle(),
        WS_OVERLAPPEDWINDOW,
        CW_USEDEFAULT,
        CW_USEDEFAULT,
        GetWidth(),
        GetHeight(),
        nullptr,        // We have no parent window.
        nullptr,        // We aren't using menus.
        GetAppInstance(),
        this);

    ShowWindow(Hwnd, GetAppCmdShow());

    // Main sample loop.
    MSG msg = {};
    while (msg.message != WM_QUIT)
    {
        // Process any messages in the queue.
        if (PeekMessage(&msg, NULL, 0, 0, PM_REMOVE))
        {
            TranslateMessage(&msg);
            DispatchMessage(&msg);
        }
    }
}

void FWin32Application::InitializePipeline()
{
    UINT dxgiFactoryFlags = 0;

    ComPtr<IDXGIFactory4> Factory;
    ThrowIfFailed(CreateDXGIFactory2(dxgiFactoryFlags, IID_PPV_ARGS(&Factory)));

    
}

LRESULT FWin32Application::WindowProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
{
    FWin32Application* App = reinterpret_cast<FWin32Application*>(GetWindowLongPtr(hWnd, GWLP_USERDATA));

    switch (message)
    {
    case WM_CREATE:
        {
            LPCREATESTRUCT pCreateStruct = reinterpret_cast<LPCREATESTRUCT>(lParam);
            SetWindowLongPtr(hWnd, GWLP_USERDATA, reinterpret_cast<LONG_PTR>(pCreateStruct->lpCreateParams));
            return 0;
        }
    case WM_KEYDOWN:
    case WM_KEYUP: return 0;
    case WM_PAINT:
        {
           if(App != nullptr) App->Update();
           break;
        }
    case WM_DESTROY:
        {
            PostQuitMessage(0);
            return 0;
        }
    }
    
    // Handle any messages the switch statement didn't.
    return DefWindowProc(hWnd, message, wParam, lParam);
}

#endif
